package product_manage

import (
	"github.com/gin-gonic/gin"
	"go-admin/app/admin/models/m_product_manage"
	"net/http"
)

// ProductList 查询产品列表
// @Summary 查询产品列表
// @Tags 产品管理
// @Produce json
// @Accept json
// @Param request body m_product_manage.ProductListReq true "查询产品列表模型"
// @Success 200 {object} m_product_manage.ProductListResp
// @Router /api/v1/product/list [post]
func ProductList(c *gin.Context) {
	var req m_product_manage.ProductListReq

	// 解析请求参数
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
		return
	}

	// 调用服务层的 GetProductList 函数获取数据
	resp, err := m_product_manage.GetProductList(c.Request.Context(), req)
	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{"error": "查询失败"})
		return
	}

	// 返回查询结果
	c.JSON(http.StatusOK, resp)
}

// CreateProduct 新建产品
// @Summary 新建产品
// @Tags 产品管理
// @Produce json
// @Accept json
// @Param request body m_product_manage.CreateProductReq true "新建产品模型"
// @Success 200 {object} m_product_manage.CreateProductResp
// @Router /api/v1/product/create [post]
func CreateProduct(c *gin.Context) {
	var req m_product_manage.CreateProductReq

	// 解析请求参数
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
		return
	}

	// 调用服务层的 CreateProduct 函数来处理逻辑
	resp, err := m_product_manage.CreateProduct(req)
	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{"error": "创建失败"})
		return
	}

	// 返回创建结果
	c.JSON(http.StatusOK, resp)
}

// EditProduct 编辑产品
// @Summary 编辑产品
// @Tags 产品管理
// @Produce json
// @Accept json
// @Param request body m_product_manage.EditProductReq true "编辑产品模型"
// @Success 200 {object} m_product_manage.EditProductResp
// @Router /api/v1/product/edit [post]
func EditProduct(c *gin.Context) {
	var req m_product_manage.EditProductReq

	// 解析请求参数
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
		return
	}

	// 调用服务层的 EditProduct 函数来处理逻辑
	resp, err := m_product_manage.EditProduct(req)
	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{"error": "编辑失败"})
		return
	}

	// 返回编辑结果
	c.JSON(http.StatusOK, resp)
}

// DeleteProduct 删除产品
// @Summary 删除产品
// @Tags 产品管理
// @Produce json
// @Accept json
// @Param request body m_product_manage.DeleteProductReq true "删除产品模型"
// @Success 200 {object} m_product_manage.DeleteProductResp
// @Router /api/v1/product/delete [post]
func DeleteProduct(c *gin.Context) {
	var req m_product_manage.DeleteProductReq

	// 解析请求参数
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusBadRequest, gin.H{"error": err.Error()})
		return
	}

	// 调用服务层的 DeleteProduct 函数来处理逻辑
	resp, err := m_product_manage.DeleteProduct(req)
	if err != nil {
		c.JSON(http.StatusInternalServerError, gin.H{"error": "删除失败"})
		return
	}

	// 返回删除结果
	c.JSON(http.StatusOK, resp)
}
